program calmain

  use globals
  use caltau
  use calp
  implicit none

  integer :: i, j, k, l
!  real(8) :: s                           ! 太陽定数
  real(8) :: ftop                        ! 大気上端での上向きフラックス
  real(8) :: alb                         ! アルベド
  real(8), dimension(lay) :: k1          ! 吸収係数
  real(8), dimension(lay-1) :: dtau      ! 光学的深さの分割
  real(8), dimension(lay) :: tau         ! 層間の光学的深さ
  real(8), dimension(lay-1) :: tauc      ! 気層の光学的深さ
  real(8), dimension(lay-1) :: dp        ! 圧力の分割
  real(8), dimension(lay) :: p           ! 層間の圧力
  real(8), dimension(lay-1) :: pc        ! 気層の圧力
  real(8), dimension(lay) :: fu, fd, fn  ! 上向き,下向き,正味のフラックス
  real(8), dimension(lay) :: t           ! 層間の温度
  real(8), dimension(lay-1) :: tc        ! 気層の温度
 
  ! global variables
  g = 9.8                            ! 9.8 m/s^2
  height = 5.0 * 10.0 ** 4.0         ! 50 km
  sheight = 8.0 * 10.0 ** 3.0        ! 8.0 km
  sheightab = 2.0 * 10.0 ** 3.0      ! 2.0 km
  theta = 0.0
  mu = cos(theta)
  pi = 2.0d0 * acos(0.0d0)
  sigma = 5.67 * 10 ** (- 8.0)       ! 5.67 * 10^(-8) Wm^(-2)K^(-4)
  h = 6.63 * 10.0 ** (-34.0)         ! 6.63 * 10^(-34) Js
  c = 3.0 * 10 ** 8.0                ! 3.0 * 10^8 m/s
  kap = 1.38 * 10.0 ** (-23.0)       ! 1.38 *10^(-23) J/K
  fg = 3.42 * 10.0 ** 2.0            ! 3.42 * 10^2 W/m^2 
  ft = 3.42 * 10.0 ** 2.0            ! 3.42 * 10^2 W/m^2
  dz = (1.0 / (lay - 1.0)) * height  ! 1.0 km
  rhoabs = 1.0 * 10.0 ** (-2.0)      ! 1.0 * 10^(-2) kg/m^3
  cp = 1.617 * 10.0 ** 3.0           ! 1617J/kgK               
  rhos = 1.3                         ! 1.3 kg/m^3
  dl = 1.0 * 10.0 ** (-8.0)          ! 10^(-8) m

  ! 初期値・分布
  alb = 0.3       
  do i = lay, 1, -1  
!    k1(i) = 0.00125  
!    k1(i) = 0.0125
!    k1(i) = 0.0375

    k1(i) = 0.0049
!   k1(i) = 0.049
!    k1(i) = 0.147

    ! 吸収係数は一定値に固定          
  end do

  call caltau1(k1, dtau, tau)
  call calp1(dp, p)

  !温度分布の計算  
  ftop = ft * (1.0 - alb) 

  do j = 1, lay
    fu(j) = (ftop / 2.0) * ((3.0 * tau(j) / 2.0) + 2.0)
    fd(j) = 3.0 * ftop * tau(j) / 4.0 
    fn(j) = fu(j) - fd(j)
  end do
 
  do j = 1, lay-1
    tauc(j) = (tau(j) + tau(j+1)) / 2.0
    pc(j) = (p(j) + p(j+1)) / 2.0
  end do

  do k = 1, lay
    t(k) = ((0.5 * ftop * (1.5 * tau(k) + 1.0)) / sigma) ** 0.25
  end do

  do k = 1, lay-1
    tc(k) = ((0.5 * ftop * (1.5 * tauc(k) + 1.0)) / sigma) ** 0.25
  end do

  open(unit=11, file='res1.dat', status='unknown')
    write(11, *) 50000, tau(1), p(1), t(1)
  close(unit=11, status='keep')
  open(unit=11, file='res1.dat', status='unknown', access='append')

  do l = 1, lay-1
    tc(l) = (t(l) + t(l+1)) / 2.0
    write(11, *) ((lay - l) * 1000) - 500, tauc(l), pc(l), tc(l)
  end do

  close(unit=11, status='keep')
  open(unit=11, file='res1.dat', status='unknown', access='append')
    write(11, *) 0, tau(lay), p(lay), t(lay) 
  close(unit=11, status='keep')

end
