module caltau

  use globals
  implicit none
  contains

subroutine caltau1( &
  & k1,             & !(in)
  & rho,            & !(out)
  & dtau,           & !(out)
  & tau,            & !(out)
  & tauc,           & !(out)
  & g1              & !(out)
  & )

  integer :: i, j, k, l
!  integer :: i1
  real(8), dimension(lay), intent(in) :: k1       ! 吸収係数
  real(8), dimension(lay) :: rhoab                ! 吸収気体の密度
  real(8), dimension(lay), intent(out) :: rho     ! 大気の密度
  real(8), dimension(lay-1), intent(out) :: dtau  ! 光学的深さの間隔
  real(8), dimension(lay), intent(out) :: tau     ! 光学的深さ
  real(8), dimension(lay-1), intent(out) :: tauc  ! 光学的深さ
  real(8), dimension(lay), intent(out) :: g1      ! 関数g1

  ! global variables
  g = 9.8                            ! 9.8 m/s^2
  height = 5.0 * 10.0 ** 4.0         ! 50 km
  sheight = 8.0 * 10.0 ** 3.0        ! 8.0 km
  sheightab = 2.0 * 10.0 ** 3.0      ! 2.0 km
  theta = 0.0
  mu = cos(theta)
  pi = 2.0d0 * acos(0.0d0)
  sigma = 5.67 * 10 ** (- 8.0)       ! 5.67 * 10^(-8) Wm^(-2)K^(-4)
  h = 6.63 * 10.0 ** (-34.0)         ! 6.63 * 10^(-34) Js
  c = 3.0 * 10 ** 8.0                ! 3.0 * 10^8 m/s
  kap = 1.38 * 10.0 ** (-23.0)       ! 1.38 *10^(-23) J/K
  dz = (1.0 / (lay - 1.0)) * height  ! 1.0 km
  rhoabs = 1.0 * 10.0 ** (-2.0)      ! 1.0 * 10^(-2) kg/m^3
  cp = 1.0 * 10.0 ** 3.0             ! 1000 J/kgK               
  rhos = 1.3                         ! 1.3 kg/m^3
  dl = 1.0 * 10.0 ** (-6.0)          ! 10^(-6)
  dt = (3.6 * 10.0 ** (3.0)) * 8.0   ! 8 h

  ! 初期値・分布
  do i = lay, 1, -1  
    rhoab(i) = rhoabs * exp(- (real(lay) - real(i)) * 10.0 ** 3.0 / sheight) 
    ! 吸収気体の密度は指数関数分布
    rho(i) = rhos * exp(- (real(lay) - real(i)) * 10.0 ** 3.0 / sheight) 
    ! 大気の密度は指数関数分布   
  end do
 
  ! dtau, tau の決定
  tau(1) = 0.0
  do j = 1, lay-1
    dtau(j) = ((k1(j) + k1(j+1)) / 2.0) * ((rhoab(j) + rhoab(j+1)) / 2.0) * dz
    tau(j+1) = tau(j) + dtau(j)
  end do 

  ! tauc の決定
  do k = 1, lay-1
    tauc(k) = (tau(k) + tau(k+1)) / 2.0
  end do 

  ! g1(tau) = exp(- tau / mu) の決定
  do l = 1, lay
    g1(l) = exp(- tau(l) / mu)
  end do 

  ! 確認用出力 
!   open(unit=12, file='restau.dat', status='unknown')   
!   do i1 = 1, lay 
!     write(12,*) i1, g1(i1)
!   end do
!   close(unit=12, status='keep')

end subroutine caltau1

end module caltau

